==> viqcore_flutter-1.2.0.tar.gz: lib/src/upgrader/upgrader.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher_string.dart';
import 'package:version/version.dart';

import '../app.dart';

/// A singleton class to configure the upgrade dialog.
class Upgrader {
  final UpgraderSettings settings;

  bool _displayed = false;
  bool _initCalled = false;
  bool _checked = false;

  String? _updateAvailable;
  DateTime? _lastTimeAlerted;
  String? _lastVersionAlerted;
  bool _hasAlerted = false;

  Upgrader({required this.settings});

  bool initialize() {
    //Skip Web build
    if (kIsWeb) return false;

    if (_initCalled) {
      return true;
    }

    _initCalled = true;

    _getSavedPrefs();

    return true;
  }

  String getMessage() {
    return 'A new version is available now! Please update to enjoy the latest features.';
  }

  String getTitle() {
    return settings.title ?? 'Update App?';
  }

  String getPrompt() {
    return settings.prompt ?? 'Would you like to update it now?';
  }

  String getButtonTitleLater() {
    return settings.buttonTitleLater ?? 'LATER';
  }

  String getButtonTitleUpdate() {
    return settings.buttonTitleUpdate ?? 'UPDATE NOW';
  }

  static void init(UpgraderSettings settings) {
    final upgrader = Upgrader(settings: settings);
    if (upgrader.initialize()) {
      App.getIt.registerSingleton(upgrader);
    }
  }

  static void execute(BuildContext context) {
    if (App.getIt.isRegistered<Upgrader>()) {
      App.getIt<Upgrader>().checkVersion(context);
    }
  }

  void checkVersion(BuildContext context) {
    if (!_initCalled || _displayed) return;
    if (_checked && _updateAvailable == null) return;
    _checked = true;
    if (shouldDisplayUpgrade() || true) {
      _displayed = true;
      _showDialog(context: context, title: getTitle(), message: getMessage());
    }
  }

  bool shouldDisplayUpgrade() {
    if (settings.debugDisplayAlways ||
        (settings.debugDisplayOnce && !_hasAlerted)) return true;

    if (!isUpdateAvailable()) return false;
    if (!settings.forceUpdate && isTooSoon()) return false;

    return true;
  }

  bool isTooSoon() {
    if (_lastTimeAlerted == null) {
      return false;
    }

    final lastAlertedDuration = DateTime.now().difference(_lastTimeAlerted!);
    return lastAlertedDuration.inDays < settings.daysToAlertAgain;
  }

  bool isUpdateAvailable() {
    if (_updateAvailable == null) {
      final appStoreVersion = Version.parse(settings.updateVersion!);
      final installedVersion = Version.parse(App.packageInfo.version);

      final available = appStoreVersion > installedVersion;
      _updateAvailable = available ? settings.updateVersion : null;

      if (settings.debugLogging) {
        if (kDebugMode) {
          print('upgrader: appStoreVersion: ${settings.updateVersion}');
          print(
              'upgrader: installedVersion: ${App.packageInfo.version.toString()}');
          print('upgrader: isUpdateAvailable: $available');
        }
      }
    }
    return _updateAvailable != null;
  }

  void _showDialog(
      {required BuildContext context,
      required String title,
      required String message}) {
    if (settings.debugLogging) {
      if (kDebugMode) {
        print('upgrader: showDialog title: $title');
        print('upgrader: showDialog message: $message');
      }
    }

    // Save the date/time as the last time alerted.
    saveLastAlerted();

    showDialog(
      barrierDismissible: false,
      context: context,
      builder: (BuildContext context) {
        return WillPopScope(
            onWillPop: () => Future.value(!settings.forceUpdate),
            child: AlertDialog(
              backgroundColor: Colors.white,
              title: Text(getTitle()),
              content: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Text(message),
                  Padding(
                      padding: const EdgeInsets.only(top: 15.0),
                      child: Text(getPrompt())),
                ],
              ),
              actions: <Widget>[
                Visibility(
                    visible: kDebugMode || !settings.forceUpdate,
                    child: TextButton(
                        child: Text(getButtonTitleLater()),
                        onPressed: () =>
                            onUserLater(context, shouldPop: true))),
                TextButton(
                    child: Text(getButtonTitleUpdate()),
                    onPressed: () => onUserUpdated(context, shouldPop: false)),
              ],
            ));
      },
    );
  }

  void onUserLater(BuildContext context, {required bool shouldPop}) {
    if (settings.debugLogging) {
      if (kDebugMode) {
        print('upgrader: button tapped: ${settings.buttonTitleLater}');
      }
    }

    // If this callback has been provided, call it.
    var doProcess = true;
    if (settings.onLater != null) {
      doProcess = settings.onLater!();
    }

    if (doProcess) {}

    if (shouldPop) {
      Navigator.of(context).pop();
    }
  }

  void onUserUpdated(BuildContext context, {required bool shouldPop}) {
    if (settings.debugLogging) {
      if (kDebugMode) {
        print('upgrader: button tapped: $settings.buttonTitleUpdate');
      }
    }

    // If this callback has been provided, call it.
    var doProcess = true;
    if (settings.onUpdate != null) {
      doProcess = settings.onUpdate!();
    }

    if (doProcess) {
      _sendUserToAppStore();
    }

    if (shouldPop) {
      Navigator.of(context).pop();
    }
  }

  Future<bool> clearSavedSettings() async {
    App.box.deleteAll(['lastTimeAlerted', 'lastVersionAlerted']);

    _lastTimeAlerted = null;
    _lastVersionAlerted = null;

    return true;
  }

  Future<bool> saveLastAlerted() async {
    _lastTimeAlerted = DateTime.now();
    App.box.put('lastTimeAlerted', _lastTimeAlerted.toString());

    _lastVersionAlerted = settings.updateVersion;
    App.box.put('lastVersionAlerted', _lastVersionAlerted!);

    _hasAlerted = true;
    return true;
  }

  bool _getSavedPrefs() {
    final lastTimeAlerted = App.box.get('lastTimeAlerted');
    if (lastTimeAlerted != null && lastTimeAlerted.isNotEmpty) {
      _lastTimeAlerted = DateTime.parse(lastTimeAlerted);
    }

    _lastVersionAlerted = App.box.get('lastVersionAlerted');

    return true;
  }

  void _sendUserToAppStore() {
    launchUrlString(settings.appstoreUrl);
  }
}

==> viqcore_flutter-1.2.2.tar.gz: lib/src/upgrader/upgrader.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher_string.dart';
import 'package:version/version.dart';

import '../app.dart';

/// A singleton class to configure the upgrade dialog.
class Upgrader {
  final UpgraderSettings settings;

  bool _displayed = false;
  bool _initCalled = false;
  bool _checked = false;

  String? _updateAvailable;
  DateTime? _lastTimeAlerted;
  String? _lastVersionAlerted;
  bool _hasAlerted = false;

  Upgrader({required this.settings});

  bool initialize() {
    //Skip Web build
    if (kIsWeb) return false;

    if (_initCalled) {
      return true;
    }

    _initCalled = true;

    _getSavedPrefs();

    return true;
  }

  String getMessage() {
    return 'A new version is available now! Please update to enjoy the latest features.';
  }

  String getTitle() {
    return settings.title ?? 'Update App?';
  }

  String getPrompt() {
    return settings.prompt ?? 'Would you like to update it now?';
  }

  String getButtonTitleLater() {
    return settings.buttonTitleLater ?? 'LATER';
  }

  String getButtonTitleUpdate() {
    return settings.buttonTitleUpdate ?? 'UPDATE NOW';
  }

  static void init(UpgraderSettings settings) {
    final upgrader = Upgrader(settings: settings);
    if (upgrader.initialize()) {
      App.getIt.registerSingleton(upgrader);
    }
  }

  static void execute(BuildContext context) {
    if (App.getIt.isRegistered<Upgrader>()) {
      App.getIt<Upgrader>().checkVersion(context);
    }
  }

  void checkVersion(BuildContext context) {
    if (!_initCalled || _displayed) return;
    if (_checked && _updateAvailable == null) return;
    _checked = true;
    if (shouldDisplayUpgrade()) {
      _displayed = true;
      _showDialog(context: context, title: getTitle(), message: getMessage());
    }
  }

  bool shouldDisplayUpgrade() {
    if (settings.debugDisplayAlways ||
        (settings.debugDisplayOnce && !_hasAlerted)) return true;

    if (!isUpdateAvailable()) return false;
    if (!settings.forceUpdate && isTooSoon()) return false;

    return true;
  }

  bool isTooSoon() {
    if (_lastTimeAlerted == null) {
      return false;
    }

    final lastAlertedDuration = DateTime.now().difference(_lastTimeAlerted!);
    return lastAlertedDuration.inDays < settings.daysToAlertAgain;
  }

  bool isUpdateAvailable() {
    if (_updateAvailable == null) {
      final appStoreVersion = Version.parse(settings.updateVersion!);
      final installedVersion = Version.parse(App.packageInfo.version);

      final available = appStoreVersion > installedVersion;
      _updateAvailable = available ? settings.updateVersion : null;

      if (settings.debugLogging) {
        if (kDebugMode) {
          print('upgrader: appStoreVersion: ${settings.updateVersion}');
          print(
              'upgrader: installedVersion: ${App.packageInfo.version.toString()}');
          print('upgrader: isUpdateAvailable: $available');
        }
      }
    }
    return _updateAvailable != null;
  }

  void _showDialog(
      {required BuildContext context,
      required String title,
      required String message}) {
    if (settings.debugLogging) {
      if (kDebugMode) {
        print('upgrader: showDialog title: $title');
        print('upgrader: showDialog message: $message');
      }
    }

    // Save the date/time as the last time alerted.
    saveLastAlerted();

    showDialog(
      barrierDismissible: false,
      context: context,
      builder: (BuildContext context) {
        return WillPopScope(
            onWillPop: () => Future.value(!settings.forceUpdate),
            child: AlertDialog(
              backgroundColor: Colors.white,
              title: Text(getTitle()),
              content: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Text(message),
                  Padding(
                      padding: const EdgeInsets.only(top: 15.0),
                      child: Text(getPrompt())),
                ],
              ),
              actions: <Widget>[
                Visibility(
                    visible: kDebugMode || !settings.forceUpdate,
                    child: TextButton(
                        child: Text(getButtonTitleLater()),
                        onPressed: () =>
                            onUserLater(context, shouldPop: true))),
                TextButton(
                    child: Text(getButtonTitleUpdate()),
                    onPressed: () => onUserUpdated(context, shouldPop: false)),
              ],
            ));
      },
    );
  }

  void onUserLater(BuildContext context, {required bool shouldPop}) {
    if (settings.debugLogging) {
      if (kDebugMode) {
        print('upgrader: button tapped: ${settings.buttonTitleLater}');
      }
    }

    // If this callback has been provided, call it.
    var doProcess = true;
    if (settings.onLater != null) {
      doProcess = settings.onLater!();
    }

    if (doProcess) {}

    if (shouldPop) {
      Navigator.of(context).pop();
    }
  }

  void onUserUpdated(BuildContext context, {required bool shouldPop}) {
    if (settings.debugLogging) {
      if (kDebugMode) {
        print('upgrader: button tapped: $settings.buttonTitleUpdate');
      }
    }

    // If this callback has been provided, call it.
    var doProcess = true;
    if (settings.onUpdate != null) {
      doProcess = settings.onUpdate!();
    }

    if (doProcess) {
      _sendUserToAppStore();
    }

    if (shouldPop) {
      Navigator.of(context).pop();
    }
  }

  Future<bool> clearSavedSettings() async {
    App.box.deleteAll(['lastTimeAlerted', 'lastVersionAlerted']);

    _lastTimeAlerted = null;
    _lastVersionAlerted = null;

    return true;
  }

  Future<bool> saveLastAlerted() async {
    _lastTimeAlerted = DateTime.now();
    App.box.put('lastTimeAlerted', _lastTimeAlerted.toString());

    _lastVersionAlerted = settings.updateVersion;
    App.box.put('lastVersionAlerted', _lastVersionAlerted!);

    _hasAlerted = true;
    return true;
  }

  bool _getSavedPrefs() {
    final lastTimeAlerted = App.box.get('lastTimeAlerted');
    if (lastTimeAlerted != null && lastTimeAlerted.isNotEmpty) {
      _lastTimeAlerted = DateTime.parse(lastTimeAlerted);
    }

    _lastVersionAlerted = App.box.get('lastVersionAlerted');

    return true;
  }

  void _sendUserToAppStore() {
    launchUrlString(settings.appstoreUrl);
  }
}
